@@ -1,5 +1,5 @@
 # Deejayd, a media player daemon
-# Copyright (C) 2007-2008 Mickael Royer <mickael.royer@gmail.com>
+# Copyright (C) 2007-2009 Mickael Royer <mickael.royer@gmail.com>
 #                         Alexandre Rossi <alexandre.rossi@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,21 +17,25 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 
-class SignalingComponent:
+class SignalingComponent(object):
+    SUBSCRIPTIONS = {}
 
     def __init__(self):
         self.__dispatcher = None
 
     def register_dispatcher(self, dispatcher):
         self.__dispatcher = dispatcher
+        # set internal subscription
+        for signame in self.SUBSCRIPTIONS.keys():
+            self.__dispatcher.subscribe(signame,\
+                    getattr(self, self.SUBSCRIPTIONS[signame]))
 
     def dispatch_signal(self, signal):
         if self.__dispatcher:
             self.__dispatcher._dispatch_signal(signal)
 
-    def dispatch_signame(self, signal_name):
+    def dispatch_signame(self, signal_name, attrs = {}):
         if self.__dispatcher:
-            self.__dispatcher._dispatch_signame(signal_name)
-
+            self.__dispatcher._dispatch_signame(signal_name, attrs)
 
 # vim: ts=4 sw=4 expandtab
