@@ -1,5 +1,5 @@
 # Deejayd, a media player daemon
-# Copyright (C) 2007-2008 Mickael Royer <mickael.royer@gmail.com>
+# Copyright (C) 2007-2009 Mickael Royer <mickael.royer@gmail.com>
 #                         Alexandre Rossi <alexandre.rossi@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -18,10 +18,12 @@
 
 import deejayd.interfaces
 from deejayd.interfaces import DeejaydError,\
-                               DeejaydAnswer, DeejaydKeyValue, DeejaydFileList,\
+                               DeejaydAnswer,\
+                               DeejaydKeyValue, DeejaydList,\
+                               DeejaydFileList,\
                                DeejaydMediaList, DeejaydDvdInfo
 from deejayd.ui.config import DeejaydConfig
-from deejayd import player, sources, mediadb, database
+from deejayd import mediafilters, player, sources, mediadb, database
 
 # Exception imports
 import deejayd.sources.webradio
@@ -71,6 +73,10 @@ def returns_deejaydanswer(answer_class):
                     if res == None:
                         ans.contents = True
                     elif answer_class == DeejaydMediaList:
+                        if isinstance(res, tuple):
+                            ans.set_filter(res[1])
+                            ans.set_sort(res[2])
+                            res = res[0]
                         ans.set_medias(res)
                     elif answer_class == DeejaydFileList:
                         root_dir, dirs, files = res
@@ -92,6 +98,105 @@ def returns_deejaydanswer(answer_class):
     return returns_deejaydanswer_instance
 
 
+class DeejaydStaticPlaylist(deejayd.interfaces.DeejaydStaticPlaylist):
+
+    def __init__(self, deejaydcore, pl_id, name):
+        self.deejaydcore = deejaydcore
+        self.db, self.library = deejaydcore.db, deejaydcore.audio_library
+        self.name = name
+        self.pl_id = pl_id
+
+    @returns_deejaydanswer(DeejaydMediaList)
+    def get(self, first=0, length=-1):
+        songs = self.db.get_static_medialist(self.pl_id,\
+            infos = self.library.media_attr)
+        last = length == -1 and len(songs) or int(first) + int(length)
+        return songs[int(first):last]
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def add_paths(self, paths):
+        ids = []
+        for path in paths:
+            try: medias = self.library.get_all_files(path)
+            except deejayd.mediadb.library.NotFoundException:
+                try: medias = self.library.get_file(path)
+                except NotFoundException:
+                    raise DeejaydError(_('Path %s not found in library') % path)
+            for m in medias: ids.append(m["media_id"])
+        self.add_songs(ids)
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def add_songs(self, song_ids):
+        self.db.add_to_static_medialist(self.pl_id, song_ids)
+        self.db.connection.commit()
+        self.deejaydcore._dispatch_signame('playlist.update',\
+                {"pl_id": self.pl_id})
+
+
+class DeejaydMagicPlaylist(deejayd.interfaces.DeejaydMagicPlaylist):
+    """ Magic playlist object """
+
+    def __init__(self, deejaydcore, pl_id, name):
+        self.deejaydcore = deejaydcore
+        self.db, self.library = deejaydcore.db, deejaydcore.audio_library
+        self.name = name
+        self.pl_id = pl_id
+
+    @returns_deejaydanswer(DeejaydMediaList)
+    def get(self, first=0, length=-1):
+        properties = dict(self.db.get_magic_medialist_properties(self.pl_id))
+        if properties["use-or-filter"] == "1":
+            filter = mediafilters.Or()
+        else:
+            filter = mediafilters.And()
+        if properties["use-limit"] == "1":
+            sort = [(properties["limit-sort-value"],\
+                     properties["limit-sort-direction"])]
+            limit = int(properties["limit-value"])
+        else:
+            sort, limit = [], None
+        filter.filterlist = self.db.get_magic_medialist_filters(self.pl_id)
+        songs = self.library.search(filter, sort, limit)
+        last = length == -1 and len(songs) or int(first) + int(length)
+        return (songs[int(first):last], filter, None)
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def add_filter(self, filter):
+        if filter.type != "basic":
+            raise DeejaydError(\
+                    _("Only basic filters are allowed for magic playlist"))
+        self.db.add_magic_medialist_filters(self.pl_id, [filter])
+        self.deejaydcore._dispatch_signame('playlist.update',\
+                {"pl_id": self.pl_id})
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def remove_filter(self, filter):
+        record_filters = self.db.get_magic_medialist_filters(self.pl_id)
+        new_filters = []
+        for record_filter in record_filters:
+            if not filter.equals(record_filter):
+                new_filters.append(record_filter)
+        self.db.set_magic_medialist_filters(self.name, new_filters)
+        self.deejaydcore._dispatch_signame('playlist.update',\
+                {"pl_id": self.pl_id})
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def clear_filters(self):
+        self.db.set_magic_medialist_filters(self.name, [])
+        self.deejaydcore._dispatch_signame('playlist.update',\
+                {"pl_id": self.pl_id})
+
+    @returns_deejaydanswer(DeejaydKeyValue)
+    def get_properties(self):
+        return dict(self.db.get_magic_medialist_properties(self.pl_id))
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def set_property(self, key, value):
+        self.db.set_magic_medialist_property(self.pl_id, key, value)
+        self.deejaydcore._dispatch_signame('playlist.update',\
+                {"pl_id": self.pl_id})
+
+
 class DeejaydWebradioList(deejayd.interfaces.DeejaydWebradioList):
 
     def __init__(self, deejaydcore):
@@ -115,11 +220,10 @@ class DeejaydWebradioList(deejayd.interfaces.DeejaydWebradioList):
 
     @returns_deejaydanswer(DeejaydAnswer)
     def delete_webradios(self, wr_ids):
-        for id in wr_ids:
-            try:
-                self.source.delete(int(id))
-            except sources._base.MediaNotFoundError:
-                raise DeejaydError(_('Webradio with id %d not found') % int(id))
+        ids = map(int, wr_ids)
+        try: self.source.delete(ids)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
     @returns_deejaydanswer(DeejaydAnswer)
     def clear(self):
@@ -139,20 +243,32 @@ class DeejaydQueue(deejayd.interfaces.DeejaydQueue):
         return songs[int(first):last]
 
     @returns_deejaydanswer(DeejaydAnswer)
-    def add_medias(self, paths, pos = None):
-        position = pos and int(pos) or None
-        try:
-            self.source.add_path(paths, position)
-        except sources._base.MediaNotFoundError:
-            raise DeejaydError(_('%s not found') % (paths,))
+    def add_songs(self, song_ids, pos = None):
+        p = pos and int(pos) or None
+        try: self.source.add_song(song_ids, pos = p)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
     @returns_deejaydanswer(DeejaydAnswer)
-    def load_playlists(self, names, pos=None):
+    def add_paths(self, paths, pos = None):
+        p = pos and int(pos) or None
+        try: self.source.add_path(paths, p)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def load_playlists(self, pl_ids, pos=None):
         pos = pos and int(pos) or None
-        try:
-            self.source.load_playlist(names, pos)
-        except sources._base.PlaylistNotFoundError:
-            raise DeejaydError(_('Playlist %s does not exist.') % str(names))
+        try: self.source.load_playlist(pl_ids, pos)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def move(self, ids, new_pos):
+        ids = [int(id) for id in ids]
+        try: self.source.move(ids, new_pos)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
     @returns_deejaydanswer(DeejaydAnswer)
     def clear(self):
@@ -160,93 +276,141 @@ class DeejaydQueue(deejayd.interfaces.DeejaydQueue):
 
     @returns_deejaydanswer(DeejaydAnswer)
     def del_songs(self, ids):
-        for id in ids:
-            try:
-                self.source.delete(int(id))
-            except sources._base.MediaNotFoundError:
-                raise DeejaydError(_('Song with id %d not found') % int(id))
+        ids = [int(id) for id in ids]
+        try: self.source.delete(ids)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
 
-class DeejaydPlaylist(deejayd.interfaces.DeejaydPlaylist):
+class DeejaydPlaylistMode(deejayd.interfaces.DeejaydPlaylistMode):
+    """Audio playlist mode."""
 
-    def __init__(self, deejaydcore, name=None):
+    def __init__(self, deejaydcore):
         self.deejaydcore = deejaydcore
         self.source = self.deejaydcore.sources.get_source("playlist")
-        self.name = name
 
     @returns_deejaydanswer(DeejaydMediaList)
     def get(self, first=0, length=-1):
-        try:
-            songs = self.source.get_content(playlist = self.name)
-        except sources._base.PlaylistNotFoundError:
-            raise DeejaydError(_('Playlist %s not found') % self.name)
-        else:
-            last = length == -1 and len(songs) or int(first) + int(length)
-            return songs[int(first):last]
+        songs = self.source.get_content()
+        last = length == -1 and len(songs) or int(first) + int(length)
+        return songs[int(first):last]
 
-    @returns_deejaydanswer(DeejaydAnswer)
+    @returns_deejaydanswer(DeejaydKeyValue)
     def save(self, name):
-        self.source.save(name)
+        if name == "":
+            raise DeejaydError(_("Set a playlist name"))
+        return self.source.save(name)
 
     @returns_deejaydanswer(DeejaydAnswer)
-    def add_songs(self, paths, position=None):
-        p = position and int(position) or None
-        try:
-            self.source.add_path(paths, playlist = self.name, pos = p)
-        except sources._base.MediaNotFoundError:
-            raise DeejaydError(_('%s not found') % (paths,))
-        except sources._base.PlaylistNotFoundError:
-            raise DeejaydError(_('Playlist %s not found') % self.name)
+    def add_paths(self, paths, pos=None):
+        p = pos and int(pos) or None
+        try: self.source.add_path(paths, pos = p)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
     @returns_deejaydanswer(DeejaydAnswer)
-    def loads(self, names, pos=None):
-        if self.name != None:
-            raise DeejaydError(_('Unable to load pls in a saved pls.'))
+    def add_songs(self, song_ids, pos=None):
+        p = pos and int(pos) or None
+        try: self.source.add_song(song_ids, pos = p)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def loads(self, pl_ids, pos=None):
         pos = pos and int(pos) or None
-        try:
-            self.source.load_playlist(names, pos)
-        except sources._base.PlaylistNotFoundError:
-            raise DeejaydError(_('Playlist %s does not exist.') % str(names))
+        try: self.source.load_playlist(pl_ids, pos)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
     @returns_deejaydanswer(DeejaydAnswer)
     def move(self, ids, new_pos):
-        if self.name != None:
-            raise DeejaydError(_('Unable to load pls in a saved pls.'))
         ids = [int(id) for id in ids]
-        try:
-            self.source.move(ids, int(new_pos))
-        except sources._base.MediaNotFoundError:
-            raise DeejaydError(_('song with ids %s not found') % (str(ids),))
+        try: self.source.move(ids, int(new_pos))
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
     @returns_deejaydanswer(DeejaydAnswer)
     def shuffle(self):
-        try:
-            self.source.shuffle(playlist = self.name)
-        except sources._base.PlaylistNotFoundError:
-            raise DeejaydError(_('Playlist %s does not exist.') % self.name)
+        try: self.source.shuffle()
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
     @returns_deejaydanswer(DeejaydAnswer)
     def clear(self):
-        try:
-            self.source.clear(playlist = self.name)
-        except sources._base.PlaylistNotFoundError:
-            raise DeejaydError(_('Playlist %s does not exist.') % self.name)
+        try: self.source.clear()
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
     @returns_deejaydanswer(DeejaydAnswer)
     def del_songs(self, ids):
-        name = self.name
-        for nb in ids:
-            try:
-                self.source.delete(int(nb), "id", playlist = name)
-            except sources._base.MediaNotFoundError:
-                raise DeejaydError(\
-                  _('Playlist %s does not have a song of id %d') % (name, nb))
-            except sources._base.PlaylistNotFoundError:
-                raise DeejaydError(_('Playlist %s does not exist.') % name)
+        ids = [int(id) for id in ids]
+        try: self.source.delete(ids)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
 
-class DeejaydVideo:
-    """Video management."""
+class DeejaydPanel(deejayd.interfaces.DeejaydPanel):
+
+    def __init__(self, deejaydcore):
+        self.deejaydcore = deejaydcore
+        self.source = self.deejaydcore.sources.get_source("panel")
+
+    @returns_deejaydanswer(DeejaydMediaList)
+    def get(self, first=0, length=-1):
+        songs, filters, sort = self.source.get_content()
+        last = length == -1 and len(songs) or int(first) + int(length)
+        return (songs[int(first):last], filters, sort)
+
+    @returns_deejaydanswer(DeejaydList)
+    def get_panel_tags(self):
+        return self.source.get_panel_tags()
+
+    @returns_deejaydanswer(DeejaydKeyValue)
+    def get_active_list(self):
+        return self.source.get_active_list()
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def set_active_list(self, type, pl_id=""):
+        try: self.source.set_active_list(type, pl_id)
+        except TypeError:
+            raise DeejaydError(_("Not supported type"))
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def set_panel_filters(self, tag, values):
+        try: self.source.set_panel_filters(tag, values)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def remove_panel_filters(self, tag):
+        try: self.source.remove_panel_filters(tag)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def clear_panel_filters(self):
+        self.source.clear_panel_filters()
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def set_search_filter(self, tag, value):
+        try: self.source.set_search_filter(tag, value)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def clear_search_filter(self):
+        self.source.clear_search_filter()
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def set_sorts(self, sorts):
+        try: self.source.set_sorts(sorts)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
+
+class DeejaydVideo(deejayd.interfaces.DeejaydVideo):
+    """Video mode."""
 
     def __init__(self, deejaydcore):
         self.deejaydcore = deejaydcore
@@ -254,16 +418,21 @@ class DeejaydVideo:
 
     @returns_deejaydanswer(DeejaydMediaList)
     def get(self, first = 0, length = -1):
-        videos = self.source.get_content()
+        videos, filters, sort = self.source.get_content()
         last = length == -1 and len(videos) or int(first) + int(length)
-        return videos[int(first):last]
+        return (videos[int(first):last], filters, sort)
 
     @returns_deejaydanswer(DeejaydAnswer)
     def set(self, value, type = "directory"):
         try: self.source.set(type, value)
-        except deejayd.sources._base.MediaNotFoundError:
-            raise DeejaydError(_('Directory %s not found in database') % value)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
+    @returns_deejaydanswer(DeejaydAnswer)
+    def set_sorts(self, sorts):
+        try: self.source.set_sorts(sorts)
+        except deejayd.sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
 class DeejayDaemonCore(deejayd.interfaces.DeejaydCore):
 
@@ -274,7 +443,6 @@ class DeejayDaemonCore(deejayd.interfaces.DeejaydCore):
             config = DeejaydConfig()
 
         self.db = database.init(config)
-        self.db.connect()
 
         self.player = player.init(self.db, config)
         self.player.register_dispatcher(self)
@@ -308,31 +476,39 @@ class DeejayDaemonCore(deejayd.interfaces.DeejaydCore):
     @returns_deejaydanswer(DeejaydAnswer)
     def play_toggle(self):
         if self.player.get_state() == player._base.PLAYER_PLAY:
-            self.player.pause()
+            current_media = self.player.get_playing()
+            if current_media['type'] == 'webradio':
+                # There is no point in pausing radio streams.
+                try: self.player.stop()
+                except player.PlayerError, err: raise DeejaydError(err)
+            else:
+                self.player.pause()
         else:
             try: self.player.play()
             except player.PlayerError, err:
-                raise DeejaydError(str(err))
+                raise DeejaydError(err)
 
     @returns_deejaydanswer(DeejaydAnswer)
     def stop(self):
-        self.player.stop()
+        try: self.player.stop()
+        except player.PlayerError, err:
+            raise DeejaydError(err)
 
     @returns_deejaydanswer(DeejaydAnswer)
     def previous(self):
         try: self.player.previous()
         except player.PlayerError, err:
-            raise DeejaydError(str(err))
+            raise DeejaydError(err)
 
     @returns_deejaydanswer(DeejaydAnswer)
     def next(self):
         try: self.player.next()
         except player.PlayerError, err:
-            raise DeejaydError(str(err))
+            raise DeejaydError(err)
 
     @returns_deejaydanswer(DeejaydAnswer)
-    def seek(self, pos):
-        self.player.set_position(int(pos))
+    def seek(self, pos, relative = False):
+        self.player.set_position(int(pos), relative)
 
     @returns_deejaydanswer(DeejaydMediaList)
     def get_current(self):
@@ -343,10 +519,12 @@ class DeejayDaemonCore(deejayd.interfaces.DeejaydCore):
         return medias
 
     @require_mode("playlist")
-    def get_playlist(self, name=None):
-        pls = DeejaydPlaylist(self, name)
-        pls.get()
-        return pls
+    def get_playlist(self):
+        return DeejaydPlaylistMode(self)
+
+    @require_mode("panel")
+    def get_panel(self):
+        return DeejaydPanel(self)
 
     @require_mode("webradio")
     def get_webradios(self):
@@ -361,6 +539,8 @@ class DeejayDaemonCore(deejayd.interfaces.DeejaydCore):
 
     @returns_deejaydanswer(DeejaydAnswer)
     def go_to(self, id, id_type = "id", source = None):
+        if id_type not in ("dvd_id","track","chapter","id","pos"):
+            raise DeejaydError(_("Bad value for id_type parm"))
         if id_type != "dvd_id":
             try: id = int(id)
             except ValueError:
@@ -368,36 +548,42 @@ class DeejayDaemonCore(deejayd.interfaces.DeejaydCore):
 
         try: self.player.go_to(id, id_type, source)
         except player.PlayerError, err:
-            raise DeejaydError(str(err))
+            raise DeejaydError(err)
 
     @returns_deejaydanswer(DeejaydAnswer)
     def set_volume(self, volume_value):
         self.player.set_volume(int(volume_value))
 
     @returns_deejaydanswer(DeejaydAnswer)
-    def set_option(self, option_name, option_value):
-        try: self.sources.set_option(option_name, int(option_value))
-        except KeyError:
-            raise DeejaydError(_('Option %s does not exist') % option_name)
+    def set_option(self, source, option_name, option_value):
+        try: self.sources.set_option(source, option_name, option_value)
+        except sources.UnknownSourceException:
+            raise DeejaydError(_('Mode %s not supported') % source)
+        except sources._base.SourceError, ex:
+            raise DeejaydError(str(ex))
 
     @returns_deejaydanswer(DeejaydAnswer)
     def set_mode(self, mode_name):
-        try:
-            self.sources.set_source(mode_name)
+        try: self.sources.set_source(mode_name)
         except sources.UnknownSourceException:
-            raise DeejaydError(_('Unknown mode: %s') % mode_name)
+            raise DeejaydError(_('Mode %s not supported') % mode_name)
 
     @returns_deejaydanswer(DeejaydKeyValue)
     def get_mode(self):
         av_sources = self.sources.get_available_sources()
         modes = {}
-        for s in self.sources.sources_list:
+        for s in self.sources.get_all_sources():
             modes[s] = s in av_sources or 1 and 0
         return modes
 
     @returns_deejaydanswer(DeejaydAnswer)
     def set_player_option(self, name, value):
-        try: self.player.set_option(name, int(value))
+        if name != "aspect_ratio":
+            try: value = int(value)
+            except (ValueError,TypeError):
+                raise DeejaydError(_("Param value is not an int"))
+
+        try: self.player.set_option(name, value)
         except KeyError:
             raise DeejaydError(_("Option %s does not exist") % name)
         except NotImplementedError:
@@ -421,29 +607,79 @@ class DeejayDaemonCore(deejayd.interfaces.DeejaydCore):
         return dict(ans)
 
     @returns_deejaydanswer(DeejaydKeyValue)
-    def update_audio_library(self, sync = False):
-        return {'audio_updating_db': self.audio_library.update(sync)}
+    def update_audio_library(self, force = False, sync = False):
+        return {'audio_updating_db': self.audio_library.update(force, sync)}
 
     @require_mode("video")
     @returns_deejaydanswer(DeejaydKeyValue)
-    def update_video_library(self, sync = False):
+    def update_video_library(self, force = False, sync = False):
         if not self.video_library:
             raise DeejaydError(_("Video mode disabled"))
-        return {'video_updating_db': self.video_library.update(sync)}
+        return {'video_updating_db': self.video_library.update(force, sync)}
+
+    @returns_deejaydanswer(DeejaydKeyValue)
+    def create_recorded_playlist(self, name, type):
+        if name == "":
+            raise DeejaydError(_("Set a playlist name"))
+        # first search if this pls already exist
+        try: self.db.get_medialist_id(name, type)
+        except ValueError: pass
+        else: # pls already exists
+            raise DeejaydError(_("This playlist already exists"))
+
+        if type == "static":
+            pl_id = self.db.set_static_medialist(name, [])
+        elif type == "magic":
+            pl_id = self.db.set_magic_medialist_filters(name, [])
+            pl = DeejaydMagicPlaylist(self, pl_id, name)
+            # set default properties for this playlist
+            default = {
+                    "use-or-filter": "0",
+                    "use-limit": "0",
+                    "limit-value": "50",
+                    "limit-sort-value": "title",
+                    "limit-sort-direction": "ascending"
+                    }
+            for (k, v) in default.items():
+                pl.set_property(k, v)
+        self._dispatch_signame('playlist.listupdate')
+        return {"pl_id": pl_id, "name": name, "type": type}
+
+    def get_recorded_playlist(self, id, name = "", type = "static"):
+        try: pl_id, name, type = self.db.is_medialist_exists(id)
+        except TypeError:
+            raise DeejaydError(_("Playlist with id %s not found.") % str(id))
+        if type == "static":
+            return DeejaydStaticPlaylist(self, pl_id, name)
+        elif type == "magic":
+            return DeejaydMagicPlaylist(self, pl_id, name)
 
-    @require_mode("playlist")
     @returns_deejaydanswer(DeejaydAnswer)
-    def erase_playlist(self, names):
-        for name in names:
-            self.sources.get_source("playlist").rm(name)
+    def erase_playlist(self, ids):
+        for id in ids:
+            self.db.delete_medialist(id)
+            self._dispatch_signame('playlist.listupdate')
 
-    @require_mode("playlist")
     @returns_deejaydanswer(DeejaydMediaList)
     def get_playlist_list(self):
-        plname_list = []
-        for plname in self.sources.get_source('playlist').get_list():
-            plname_list.append({'name': plname})
-        return plname_list
+        return [{"name": pl, "id":id, "type":type}\
+            for (id, pl, type) in self.db.get_medialist_list() if not \
+            pl.startswith("__") or not pl.endswith("__")]
+
+    @returns_deejaydanswer(DeejaydAnswer)
+    def set_media_rating(self, media_ids, rating, type = "audio"):
+        if int(rating) not in range(0, 5):
+            raise DeejaydError(_("Bad rating value"))
+
+        try: library = getattr(self, type+"_library")
+        except AttributeError:
+            raise DeejaydError(_('Type %s is not supported') % (type,))
+        for id in media_ids:
+            try: library.set_file_info(int(id), "rating", rating)
+            except TypeError:
+                raise DeejaydError(_("%s library not activated") % type)
+            except deejayd.mediadb.library.NotFoundException:
+                raise DeejaydError(_("File with id %s not found") % str(id))
 
     @returns_deejaydanswer(DeejaydFileList)
     def get_audio_dir(self,dir = None):
@@ -454,13 +690,27 @@ class DeejayDaemonCore(deejayd.interfaces.DeejaydCore):
 
         return dir, contents['dirs'], contents['files']
 
-    @returns_deejaydanswer(DeejaydFileList)
-    def audio_search(self, search_txt, type = 'all'):
-        try: list = self.audio_library.search(type, search_txt)
+    @returns_deejaydanswer(DeejaydKeyValue)
+    def get_audio_cover(self,media_id):
+        try: cover = self.audio_library.get_cover(media_id)
         except deejayd.mediadb.library.NotFoundException:
-            raise DeejaydError(_('Type %s is not supported') % (type,))
+            raise DeejaydError(_('Cover not found'))
+        return cover
 
-        return None, [], list
+    @returns_deejaydanswer(DeejaydMediaList)
+    def audio_search(self, pattern, type = 'all'):
+        if type not in ('all','title','genre','filename','artist','album'):
+            raise DeejaydError(_('Type %s is not supported') % (type,))
+        if type == "all":
+            filter = mediafilters.Or()
+            for tag in ('title','genre','artist','album'):
+                filter.combine(mediafilters.Contains(tag, pattern))
+        else:
+            filter = mediafilters.Contains(type, pattern)
+        songs = self.audio_library.search(filter,\
+                mediafilters.DEFAULT_AUDIO_SORT)
+
+        return songs
 
     @require_mode("video")
     @returns_deejaydanswer(DeejaydFileList)
@@ -487,5 +737,9 @@ class DeejayDaemonCore(deejayd.interfaces.DeejaydCore):
     def get_dvd_content(self):
         return self.sources.get_source("dvd").get_content()
 
+    @returns_deejaydanswer(DeejaydList)
+    def mediadb_list(self, tag, filter):
+        return [x[0] for x in self.db.list_tags(tag, filter)]
+
 
 # vim: ts=4 sw=4 expandtab
